# Copyright 2026 Terrene Foundation
# SPDX-License-Identifier: Apache-2.0

"""
EATP Database Migrations.

Migration scripts for the Enterprise Agent Trust Protocol (EATP)
database schema updates.

Migrations are designed to be:
- Idempotent: Safe to run multiple times
- Backward Compatible: Old code continues to work during migration
- Transactional: Changes rolled back on failure
"""
